package com.ruoyi.ss.vipLevel.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import com.ruoyi.ss.vipLevel.domain.VipLevelQuery;
import com.ruoyi.ss.vipLevel.domain.VipLevelVO;
import com.ruoyi.ss.vipLevel.mapper.VipLevelMapper;
import com.ruoyi.ss.vipLevel.service.VipLevelAssembler;
import com.ruoyi.ss.vipLevel.service.VipLevelService;
import com.ruoyi.ss.vipLevel.service.VipLevelValidator;
import com.ruoyi.ss.vipLevelSku.domain.VipLevelSkuVO;
import com.ruoyi.ss.vipLevelSku.service.VipLevelSkuService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Collections;
import java.util.List;

/**
 * 会员等级Service业务层处理
 *
 * @author ruoyi
 * @date 2024-12-09
 */
@Service
public class VipLevelServiceImpl implements VipLevelService
{
    @Autowired
    private VipLevelMapper vipLevelMapper;

    @Autowired
    private VipLevelValidator vipLevelValidator;

    @Autowired
    private VipLevelSkuService vipLevelSkuService;

    @Autowired
    private VipLevelAssembler vipLevelAssembler;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询会员等级
     *
     * @param id 会员等级主键
     * @return 会员等级
     */
    @Override
    public VipLevelVO selectVipLevelById(Long id)
    {
        return vipLevelMapper.selectVipLevelById(id);
    }

    /**
     * 查询会员等级列表
     *
     * @param vipLevel 会员等级
     * @return 会员等级
     */
    @Override
    public List<VipLevelVO> selectVipLevelList(VipLevelQuery vipLevel)
    {
        return vipLevelMapper.selectVipLevelList(vipLevel);
    }

    /**
     * 新增会员等级
     *
     * @param vipLevel 会员等级
     * @return 结果
     */
    @Override
    public int insertVipLevel(VipLevelVO vipLevel) {

        Integer result = transactionTemplate.execute(status -> {
            // 主表
            vipLevel.setCreateTime(DateUtils.getNowDate());
            int insert = vipLevelMapper.insertVipLevel(vipLevel);

            if (insert == 1) {
                // SKU表
                this.batchUpdateSku(vipLevel);

                // 后校验
                this.afterCheck(vipLevel.getId());
            }

            return insert;
        });

        return result == null ? 0 : result;
    }

    private void afterCheck(Long id) {
        VipLevelVO vo = this.selectVipLevelById(id);
        List<VipLevelVO> list = Collections.singletonList(vo);
        vipLevelAssembler.assembleSkuList(list);
        vipLevelSkuService.afterCheck(vo);
    }

    /**
     * 修改会员等级
     *
     * @param vipLevel 会员等级
     * @return 结果
     */
    @Override
    public int updateVipLevel(VipLevelVO vipLevel) {
        Integer result = transactionTemplate.execute(status -> {
            // 主表
            int update = vipLevelMapper.updateVipLevel(vipLevel);

            if (update == 1) {
                // SKU表
                this.batchUpdateSku(vipLevel);

                // 后校验
                this.afterCheck(vipLevel.getId());
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    private int batchUpdateSku(VipLevelVO level) {
        if (level == null) {
            return 0;
        }
        // 分离出新旧数据
        List<VipLevelSkuVO> newList = level.getSkuList();
        if (CollectionUtils.isNotEmptyElement(newList)) {
            for (VipLevelSkuVO sku : newList) {
                sku.setLevelId(level.getId());
            }
        }
        List<VipLevelSkuVO> oldList = vipLevelSkuService.selectByLevelId(level.getId());
        DiffListVO<VipLevelSkuVO, VipLevelSkuVO> diff = CollectionUtils.convertToDiffList(newList, oldList, VipLevelSkuVO::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            result += vipLevelSkuService.batchInsert(diff.getAddList());
        }
        if (diff.getUpdateCount() > 0) {
            result += vipLevelSkuService.batchUpdate(diff.getUpdateList());
        }
        if (diff.getDelCount() > 0) {
            result += vipLevelSkuService.deleteVipLevelSkuByIds(CollectionUtils.map(diff.getDelList(), VipLevelSkuVO::getId));
        }

        return result;
    }

    /**
     * 批量删除会员等级
     *
     * @param ids 需要删除的会员等级主键
     * @return 结果
     */
    @Override
    public int deleteVipLevelByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return 0;
        }
        vipLevelValidator.beforeDel(ids);
        return vipLevelMapper.deleteVipLevelByIds(ids);
    }

    /**
     * 删除会员等级信息
     *
     * @param id 会员等级主键
     * @return 结果
     */
    @Override
    public int deleteVipLevelById(Long id)
    {
        return vipLevelMapper.deleteVipLevelById(id);
    }

    @Override
    public List<VipLevelVO> selectByIds(List<Long> ids) {
        if (CollectionUtils.isEmptyElement(ids)) {
            return Collections.emptyList();
        }
        VipLevelQuery query = new VipLevelQuery();
        query.setIds(ids);
        return this.selectVipLevelList(query);
    }
}
